﻿DELIMITER $$

DROP PROCEDURE IF EXISTS `upgrade_history_schema` $$
CREATE DEFINER=`dharma_user`@`localhost` PROCEDURE `upgrade_history_schema`()
BEGIN



  /* find out the build version so we know if we need to do some schema altering */

  SET @need_alters = 0 ;

  SELECT build_info.version
  INTO @current_version
  FROM build_info
  WHERE dri_time IN (SELECT max(dri_time) FROM build_info);

  
  SELECT CASE WHEN (@current_version LIKE '7.0%') OR (@current_version LIKE '7.10%')
                   THEN 1
                   ELSE 0
         END
  INTO @need_alters;

  /* only do this if version is < 7.11 !!! */
  IF @need_alters = 1 THEN

      SELECT CONCAT('Schema is at version ', @current_version, '. Updating.. (This may take some time)') AS "Message";

      /* drop constraint, if it exists*/
      IF EXISTS (SELECT NULL FROM information_schema.TABLE_CONSTRAINTS
                 WHERE CONSTRAINT_SCHEMA = DATABASE() AND CONSTRAINT_NAME = 'fk_hist_rstep2parent') THEN

        ALTER TABLE runstep_history DROP FOREIGN KEY fk_hist_rstep2parent;
      END IF;


      /** create index if not there already */
      IF EXISTS (SELECT NULL FROM information_schema.statistics
                 WHERE INDEX_SCHEMA = DATABASE() AND index_name = 'idx_hist_prop_runhist_id') THEN

          ALTER TABLE property_history
                DROP INDEX idx_hist_prop_runhist_id;

      END IF;

      CREATE INDEX idx_hist_prop_runhist_id
             ON property_history(run_hist_id);


      /* replace some of the foreign keys generated by hibernate
        with the same foreign keys, but with DELETE CASCADE */
      IF EXISTS (SELECT NULL FROM information_schema.TABLE_CONSTRAINTS
                 WHERE CONSTRAINT_SCHEMA = DATABASE() AND CONSTRAINT_NAME = 'fk_hist_rstep2run') THEN
         ALTER TABLE runstep_history
               DROP FOREIGN KEY fk_hist_rstep2run;
      END IF;

      ALTER TABLE runstep_history
            ADD CONSTRAINT fk_hist_rstep2run
            FOREIGN KEY (run_history_id)
            REFERENCES run_history(oid)
            ON DELETE CASCADE;

      IF EXISTS (SELECT NULL FROM information_schema.TABLE_CONSTRAINTS
                 WHERE CONSTRAINT_SCHEMA = DATABASE() AND CONSTRAINT_NAME = 'fk_hist_prop2rstep') THEN

         ALTER TABLE property_history
               DROP FOREIGN KEY fk_hist_prop2rstep ;
      END IF;

      ALTER TABLE property_history
            ADD CONSTRAINT fk_hist_prop2rstep
            FOREIGN KEY (runstep_hist_id)
            REFERENCES runstep_history(oid)
            ON DELETE CASCADE;


  ELSE
    SELECT CONCAT('Schema is at version ' , @current_version, '. No update is required.') as "Message";

  END IF;

END $$

DELIMITER ;
